@@ -577,8 +577,8 @@ def class_attributes(classobj,of_class=None,
 
 def inst_attributes(obj,of_class=None):
 
-    """ Find all instance attributes of obj that are instances of of_class
-        and return them as dictionary.
+    """ Find all instance attributes of obj that are instances of
+        of_class and return them as dictionary.
 
     """
     d = {}
