@@ -8,16 +8,19 @@
     or contact the author. All Rights Reserved.
 """
 import string, types, glob, os, sys
-from distutils.errors import DistutilsError, DistutilsExecError, CompileError
+from distutils.errors import \
+     DistutilsError, DistutilsExecError, CompileError, CCompilerError
 from distutils.core import setup, Extension, Command
 from distutils.sysconfig import get_config_h_filename, parse_config_h
 from distutils.msvccompiler import MSVCCompiler
 from distutils.util import execute
+from distutils.dir_util import remove_tree
 from distutils.spawn import spawn, find_executable
 from distutils.dist import Distribution
 from distutils.command.config import config
 from distutils.command.build import build
 from distutils.command.build_ext import build_ext
+from distutils.command.build_clib import build_clib
 from distutils.command.build_py import build_py
 from distutils.command.install_data import install_data
 
@@ -170,6 +173,27 @@ class mx_Distribution(Distribution):
     def has_unixlibs(self):
         return self.unixlibs and len(self.unixlibs) > 0
 
+#
+# mx Extension class
+#
+
+class mx_Extension(Extension):
+
+    """ Extension class which allows specifying whether the extension
+        is required to build or optional.
+        
+    """
+    # Is this Extension required to build or can we issue a Warning in
+    # case it fails to build and continue with the remaining build
+    # process ?
+    required = 1
+
+    def __init__(self, *args, **kws):
+        if kws.has_key('required'):
+            self.required = kws['required']
+            del kws['required']
+        apply(Extension.__init__, (self,) + args, kws)
+
 #
 # mx Build command
 #
@@ -192,6 +216,69 @@ class mx_build(build):
                     ('build_scripts', build.has_scripts),
                    ]
 
+#
+# mx Build C Lib command
+#
+
+class mx_build_clib(build_clib):
+
+    """ build_clib command which builds the libs using
+        separate temp dirs
+        
+    """
+    def build_library(self, lib_name, build_info):
+
+        # Build each extension in its own subdir of build_temp (to
+        # avoid accidental sharing of object files between extensions
+        # having the same name, e.g. mxODBC.o).
+        build_temp_base = self.build_temp
+        self.build_temp = os.path.join(build_temp_base, lib_name)
+
+        try:
+
+            #
+            # XXX Copied from the original build_clib !!!
+            #
+
+            sources = build_info.get('sources')
+            if sources is None or \
+               type(sources) not in (types.ListType, types.TupleType):
+                raise DistutilsSetupError, \
+                      ("in 'libraries' option (library '%s'), " +
+                       "'sources' must be present and must be " +
+                       "a list of source filenames") % lib_name
+            sources = list(sources)
+
+            self.announce("building '%s' library" % lib_name)
+
+            # First, compile the source code to object files in the
+            # library directory.  (This should probably change to
+            # putting object files in a temporary build directory.)
+            macros = build_info.get('macros')
+            include_dirs = build_info.get('include_dirs')
+            objects = self.compiler.compile(sources,
+                                            output_dir=self.build_temp,
+                                            macros=macros,
+                                            include_dirs=include_dirs,
+                                            debug=self.debug)
+
+            # Now "link" the object files together into a static library.
+            # (On Unix at least, this isn't really linking -- it just
+            # builds an archive.  Whatever.)
+            self.compiler.create_static_lib(objects, lib_name,
+                                            output_dir=self.build_clib,
+                                            debug=self.debug)
+            
+            
+        finally:
+            # Cleanup local changes to the configuration
+            self.build_temp = build_temp_base
+        
+    def build_libraries(self, libraries):
+
+        for (lib_name, build_info) in libraries:
+            self.build_library(lib_name, build_info)
+
 #
 # mx Build Extensions command
 #
@@ -212,6 +299,35 @@ class mx_build_ext(build_ext):
         self.run_command('mx_autoconf')
         build_ext.run(self)
 
+    def build_extension(self, ext):
+
+        # Build each extension in its own subdir of build_temp (to
+        # avoid accidental sharing of object files between extensions
+        # having the same name, e.g. mxODBC.o).
+        build_temp_base = self.build_temp
+        extpath = string.join(string.split(ext.name, '.'), os.sep)
+        self.build_temp = os.path.join(build_temp_base, extpath)
+
+        try:
+            
+            # Skip extensions which cannot be built if the required
+            # option is given and set to false.
+            required = not hasattr(ext, 'required') or ext.required
+            if required:
+                build_ext.build_extension(self, ext)
+            else:
+                try:
+                    build_ext.build_extension(self, ext)
+                except (CCompilerError, DistutilsError), why:
+                    self.announce(
+                        '*** WARNING: Building of extension "%s" '
+                        'failed: %s' %
+                        (ext.name, sys.exc_info()[1]))
+                    
+        finally:
+            # Cleanup local changes to the configuration
+            self.build_temp = build_temp_base
+
 #
 # mx Build Python command
 #
@@ -435,51 +551,63 @@ class mx_build_unixlib(Command):
 
     def build_unixlib(self, unixlib):
 
-        # Get options
-        configure = unixlib.configure
-        configure_options = unixlib.configure_options
-        make_options = unixlib.make_options
-        sourcetree = unixlib.sourcetree
-        buildtree = os.path.join(self.build_temp, sourcetree)
-        libfiles = unixlib.libfiles
-        if not libfiles:
-            raise DistutilsError, \
-                  'no libfiles defined for unixlib "%s"' % unixlib.name
-        if self.verbose:
-            self.announce('building C lib %s in %s' % (unixlib.libname,
-                                                       buildtree))
-        # Prepare build
-        if self.verbose:
-            self.announce('preparing build of %s' % unixlib.libname)
-        self.mkpath(buildtree)
-        self.copy_tree(sourcetree, buildtree)
-        
-        # Run configure to build the Makefile
-        if not os.path.exists(os.path.join(buildtree, self.makefile)) or \
-           self.force:
-            self.run_configure(configure_options,
-                               dir=buildtree,
-                               configure=configure)
-        elif self.verbose:
-            self.announce('skipping configure: %s is already configured' %
-                          unixlib.libname)
+        # Build each lib in its own subdir of build_temp (to
+        # avoid accidental sharing of object files)
+        build_temp_base = self.build_temp
+        libpath = unixlib.libname
+        self.build_temp = os.path.join(build_temp_base, libpath)
 
-        # Run make
-        self.run_make(dir=buildtree,
-                      make=self.make,
-                      options=make_options)
+        try:
 
-        # Copy libs to destinations
-        for sourcefile, destination in libfiles:
-            if not destination:
-                continue
-            sourcefile = os.path.join(self.build_temp, sourcefile)
-            destination = os.path.join(self.build_lib, destination)
-            if not os.path.exists(sourcefile):
-                raise CompileError, \
-                      'library "%s" failed to build' % sourcefile
-            self.mkpath(destination)
-            self.copy_file(sourcefile, destination)
+            # Get options
+            configure = unixlib.configure
+            configure_options = unixlib.configure_options
+            make_options = unixlib.make_options
+            sourcetree = unixlib.sourcetree
+            buildtree = os.path.join(self.build_temp, sourcetree)
+            libfiles = unixlib.libfiles
+            if not libfiles:
+                raise DistutilsError, \
+                      'no libfiles defined for unixlib "%s"' % unixlib.name
+            if self.verbose:
+                self.announce('building C lib %s in %s' % (unixlib.libname,
+                                                           buildtree))
+            # Prepare build
+            if self.verbose:
+                self.announce('preparing build of %s' % unixlib.libname)
+            self.mkpath(buildtree)
+            self.copy_tree(sourcetree, buildtree)
+
+            # Run configure to build the Makefile
+            if not os.path.exists(os.path.join(buildtree, self.makefile)) or \
+               self.force:
+                self.run_configure(configure_options,
+                                   dir=buildtree,
+                                   configure=configure)
+            elif self.verbose:
+                self.announce('skipping configure: %s is already configured' %
+                              unixlib.libname)
+
+            # Run make
+            self.run_make(dir=buildtree,
+                          make=self.make,
+                          options=make_options)
+
+            # Copy libs to destinations
+            for sourcefile, destination in libfiles:
+                if not destination:
+                    continue
+                sourcefile = os.path.join(self.build_temp, sourcefile)
+                destination = os.path.join(self.build_lib, destination)
+                if not os.path.exists(sourcefile):
+                    raise CompileError, \
+                          'library "%s" failed to build' % sourcefile
+                self.mkpath(destination)
+                self.copy_file(sourcefile, destination)
+
+        finally:
+            # Cleanup local changes to the configuration
+            self.build_temp = build_temp_base
 
     def build_unixlibs(self, unixlibs):
 
@@ -670,6 +798,7 @@ def run_setup(configurations):
                   'build_unixlib': mx_build_unixlib,
                   'mx_autoconf': mx_autoconf,
                   'build_ext': mx_build_ext,
+                  'build_clib': mx_build_clib,
                   'build_py': mx_build_py,
                   'install_data': mx_install_data,
                   'uninstall': mx_uninstall,
